%% Load data from the file TOUGHout_3D...

if exist('out.mat')==2 && exist('mesh_data.mat')==2
    load out
else
    [out times]=READ_DATA('TOUGHout_3D');
    save out out times
end

%% load mesh data already saved by READ_DATA

load mesh_data

X=Coor{1,1};
Y=Coor{1,2};
Z=Coor{1,3};


%% Let's plot the pressure:
% the simulation resulted in 1 single printout the out is a 1x1 cell
% the variable pressure is the column number 4
% see ReadMe file for more infos on loading the data

Pressure=out{1,1}(:,4);
x_mesh=out{1,1}(:,1); % also val{1,1} can be used from file mesh_data.mat
y_mesh=out{1,1}(:,2); % also val{1,2} can be used from file mesh_data.mat
z_mesh=out{1,1}(:,3); % also val{1,3} can be used from file mesh_data.mat


%% Plotting at fixed depth refining the mesh - PLOT XY:
% we need to find in this block-by-block vector only the blocks that are 
% at fixed depth. Let's say we eant to plot at z=-1810 (not that this
% number has to be one coordinate of the original TOUGH2 mesh)

Press_at_fix_z=Pressure(z_mesh==-1810);
x_mesh_at_fix_z=x_mesh(z_mesh==-1810);
y_mesh_at_fix_z=y_mesh(z_mesh==-1810);


xlin=linspace(min(X),max(X),300); %linear spacing betwwen min(X) and max(X)
ylin=linspace(min(Y),max(Y),300); %linear spacing between min(Y) and max(Y)

[Xcoord,Ycoord]=meshgrid(xlin,ylin);

var_image=griddata(x_mesh_at_fix_z,y_mesh_at_fix_z,Press_at_fix_z,Xcoord,Ycoord);

figure(1)
image(xlin/1000,ylin/1000,var_image/1.e6,'Cdatamapping','scaled')
axis image
set(gca,'YDir','normal')
colorbar
xlabel('X (km)')
ylabel('Y (km)')
title('Pressure (MPa)')

%% Plotting at fixed X refining the mesh - PLOT YZ:
% we need to find in this block-by-block vector only the blocks that are 
% at fixed x. Let's say we eant to plot at x=-12.5 (not that this
% number has to be one coordinate of the original TOUGH2 mesh)

Press_at_fix_x=Pressure(x_mesh==-12.5);
y_mesh_at_fix_x=y_mesh(x_mesh==-12.5);
z_mesh_at_fix_x=z_mesh(x_mesh==-12.5);


ylin=linspace(min(Y),max(Y),300); %linear spacing betwwen min(Y) and max(Y)
zlin=linspace(min(Z),max(Z),300); %linear spacing between min(Z) and max(Z)

[Ycoord,Zcoord]=meshgrid(ylin,zlin);

var_image=griddata(y_mesh_at_fix_x,z_mesh_at_fix_x,Press_at_fix_x,Ycoord,Zcoord);

figure(2)
image(ylin/1000,zlin/1000,var_image/1.e6,'Cdatamapping','scaled')
axis image
set(gca,'YDir','normal','Xlim',[-2 2])
colorbar
xlabel('Y (km)')
ylabel('Z (km)')
title('Pressure (MPa)')

%% Using command slice to plot multiple planes at the same time:
% this is a little complex, since create 3D vector in MATLAB
% and use command griddata3 and slice

[Xco,Yco,Zco]=meshgrid(-500:50:500,-500:50:500,-1900:50:-1400);
var_image=griddata3(x_mesh,y_mesh,z_mesh,Pressure,Xco,Yco,Zco);

xslice=[12.5]; yslice=[450]; zslice=[-1810];

figure(3)
hslice=slice(Xco,Yco,Zco,var_image/1.e6,xslice,yslice,zslice);
set(hslice,'FaceColor','interp','EdgeColor','none')
set(gca,'XLim',[-500 500],'ZLim',[-1900 -1400])
colorbar
xlabel('X(m)')
ylabel('Y(m)')
zlabel('Z(m)')


figure(4)
hslice=slice(Xco,Yco,Zco,var_image/1.e6,xslice,yslice,zslice);
set(hslice,'FaceColor','interp','EdgeColor','none')
view([-37.5+90,30])
set(gca,'XLim',[-500 500],'ZLim',[-1900 -1400])
colorbar
xlabel('X(m)')
ylabel('Y(m)')
zlabel('Z(m)')



